@@ -105,6 +105,7 @@ import Control.Monad (liftM, when, unless)
 import Control.Monad.Trans.State.Strict (get, put)
 import Data.Functor.Identity (Identity, runIdentity)
 import Foreign.C.Error (Errno(Errno), ePIPE)
+import GHC.Exts (build)
 import Pipes
 import Pipes.Core
 import Pipes.Internal
@@ -414,12 +415,13 @@ filterM predicate = for cat $ \a -> do
 > take (min m n) = take m >-> take n
 -}
 take :: Monad m => Int -> Pipe a a m ()
-take = loop where
-  loop 0 = return () 
-  loop n = do 
-    a <- await
-    yield a
-    loop (n-1)
+take = go
+  where
+    go 0 = return () 
+    go n = do 
+        a <- await
+        yield a
+        go (n-1)
 {-# INLINABLE take #-}
 
 {-| @(takeWhile p)@ allows values to pass downstream so long as they satisfy
@@ -467,12 +469,12 @@ takeWhile' predicate = go
 > drop (m + n) = drop m >-> drop n
 -}
 drop :: Monad m => Int -> Pipe a a m r
-drop = loop
+drop = go
   where
-    loop 0 = cat
-    loop n =  do
-      await
-      loop (n-1)
+    go 0 = cat
+    go n =  do
+        await
+        go (n-1)
 {-# INLINABLE drop #-}
 
 {-| @(dropWhile p)@ discards values going downstream until one violates the
@@ -611,7 +613,7 @@ loop k = for cat (every . k)
     Use these to fold the output of a 'Producer'.  Many of these folds will stop
     drawing elements if they can compute their result early, like 'any':
 
->>> P.any null P.stdinLn
+>>> P.any Prelude.null P.stdinLn
 Test<Enter>
 ABC<Enter>
 <Enter>
@@ -813,14 +815,15 @@ product = fold (*) 1 id
 
 -- | Convert a pure 'Producer' into a list
 toList :: Producer a Identity () -> [a]
-toList = go
+toList prod0 = build (go prod0)
   where
-    go p = case p of
+    go prod cons nil =
+      case prod of
         Request v _  -> closed v
-        Respond a fu -> a:go (fu ())
-        M         m  -> go (runIdentity m)
-        Pure    _    -> []
-{-# INLINABLE toList #-}
+        Respond a fu -> cons a (go (fu ()) cons nil)
+        M         m  -> go (runIdentity m) cons nil
+        Pure    _    -> nil
+{-# INLINE toList #-}
 
 {-| Convert an effectful 'Producer' into a list
 
